! Sample program for gtool_history/gtool5 and deepconv
! 
! Solving 2-D non-divergent (barotropic) fluid system
!     d\zeta/dt  = \nu\nabla^2\zeta
!
program diffuse

  !---- ⥸塼ɤ߹ ----  
  use dc_types,      only: STRING, DP
  use gtool_historyauto, only: HistoryAutoPut, HistoryAutoAddVariable
  ! MPI
  use mpi_wrapper, only : MPIWrapperInit, MPIWrapperFinalize
  ! 
  use argset,     only: argset_init
  ! ʻ
  use gridset,    only: gridset_init, imin, imax, jmin, jmax, kmin, kmax, nx, ny, nz
  ! 
  use axesset,    only: axesset_init, x_X, z_Z
  ! ե
  use fileset,    only: fileset_init
  ! 
  use timeset,    only: timeset_init, DelTime, EndTime, StartTime
  ! ʬ黻
  use xyz_deriv_module, only: pyz_dx_xyz, xqz_dy_xyz, xyr_dz_xyz, &
    &                         xyz_dx_pyz, xyz_dy_xqz, xyz_dz_xyr
  ! 
  use xyz_bc_module, only: BoundaryXCyc_xyz, BoundaryZCyc_xyz          
  ! ե
  use HistoryFileIO, only: HistoryFileAutoOpen, HistoryFileAutoClose

  !---- ۤηػ ----  
  implicit none

  !---- ѿ ----
  real(DP), allocatable :: xyz_ZetaN(:,:,:)
  real(DP), allocatable :: xyz_ZetaA(:,:,:)
  real(DP)              :: TimeA, TimeN
  
 !---- ʪѥ᥿ ----
  real(DP), parameter :: nu=1.0         ! Ǵ
  real(DP), parameter :: sigma=0.1      ! ʬۤ礭
  real(DP), parameter :: x1=5.0d-1      ! ʬ X ɸ
  real(DP), parameter :: z1=5.0d-1      ! ʬ Z ɸ

  !---- ϥե -----------
  character(STRING) :: cfgfile

  !----  -----------
  integer :: i, k

 !----------------  ---------------------
  call MPIWrapperInit
  call argset_init( cfgfile )  !(out)
  call fileset_init( cfgfile ) !(in)
  call timeset_init( cfgfile ) !(in)
  call gridset_init( cfgfile ) !(in)
  call axesset_init( cfgfile ) !(in)
  call HistoryFileAutoOpen (cfgfile) !(in)

  call HistoryAutoAddVariable(  &
    & varname='zeta',           &  
    & dims=(/'x','y','z','t'/), &
    & longname='vorticity',     &
    & units=' ',                &        
    & xtype='double' )

 !---------------- ѿ allocate ---------------------
  allocate(xyz_ZetaN(imin:imax,jmin:jmax,kmin:kmax))
  allocate(xyz_ZetaA(imin:imax,jmin:jmax,kmin:kmax))

 !-------------------  ----------------------
  TimeN = StartTime
  TimeA = StartTime + DelTime

  do k = imin, imax
    do i = kmin, kmax
      xyz_ZetaN(i,:,k) = dexp(-((x_X(i)-x1)**2 + (z_Z(k)-z1)**2)/ (2*sigma**2))
    end do
  end do

 !------------------- ʬ ----------------------
  do while (TimeN <= EndTime)    

!   d\zeta/dt  = \nu\nabla^2\zeta
    
!    pyz_dx_xyz(xyz_A) =>  [ ( A(i,k) - A(i-1,k) ) /dx ]_(i-1(u),k)

    xyz_ZetaA = xyz_ZetaN                 &
      &        + DelTime * nu * (            &
      &              xyz_dx_pyz(pyz_dx_xyz(xyz_ZetaN)) &
      &            + xyz_dy_xqz(xqz_dy_xyz(xyz_ZetaN)) &
      &            + xyz_dz_xyr(xyr_dz_xyz(xyz_ZetaN)) &
      &          )                                   ! Euler ˡˤʬ

    call BoundaryXcyc_xyz( xyz_ZetaA ) 
    call BoundaryZCyc_xyz( xyz_ZetaA ) 

    ! 롼׽
    xyz_ZetaN = xyz_ZetaA
    TimeA = TimeA + DelTime
    TimeN = TimeA

    ! 
    call HistoryAutoPut(TimeN, 'zeta', xyz_ZetaN(1:nx,1:ny,1:nz))    
  end do
  
  !------------------- λ ----------------------
  call HistoryFileAutoClose
  call MPIWrapperFinalize
  stop

end program diffuse
